(* ::Package:: *)

(* ::Section:: *)
(*Run asir script.*)


(* ::Text:: *)
(*Wanted output must be named 'OUTPUT' in the script.*)


runasir[code___String] := Module[{tmpfile, resfile, result},
	tmpfile = MkTemp["asir", ".rr"];
	resfile = tmpfile // StringReplace[".rr" -> ".txt"];
	MkFile[tmpfile, 
		code,
		"\n\n",
		"mkfile(\"", 
		resfile,
		"\", OUTPUT);"
	];
	Run["asir -f " <> tmpfile];
	result = Get[resfile];
	DeleteFile[{tmpfile, resfile}];
	result
]


(* ::Text:: *)
(*Mathematica list to asir list*)


asirlist[mmalist_] /; Length@Dimensions@mmalist > 1 := mmalist // RightComposition[
	ToString,
	StringReplace[{ 
		"{"  ->  "[" , 
		"}"  ->  "]" 
	}],
	StringReplace[{
		", [" ->  ",\n\t["  ,
		"[["  ->  "[\n\t["  ,
		"]]"  ->  "]\n];"
	}],
	StringDelete[" "],
	Identity
]

asirlist[mmalist_] /; Length@Dimensions@mmalist == 1 := mmalist // RightComposition[
	ToString,
	StringReplace[{ 
		"{"  ->  "[" , 
		"}"  ->  "];" 
	}],
	StringDelete[" "],
	Identity
]


(* ::Section:: *)
(*Twisted form to D-module element*)


(* ::Text:: *)
(*List of {xs, dxs} left over after simplex-restriction *)


simplexvars[A_,sigma_] := Module[{varinds,sigmacomp,vars},
	varinds = Dimensions[A][[2]] // Range;
	sigmacomp = Complement[varinds, sigma];
	vars = varinds // Map[{# // x, # // dx}&] // Map[ind2sym, #, {2,2}]&;
	vars[[sigmacomp]] // Transpose
]


(* ::Text:: *)
(*A'*)


Ap[A_] := ReplacePart[
	A,
	{i_ /; i>1 , __} -> 0
]


(* ::Text:: *)
(*Length@Dimensions@rvecs > 1 to ensure syntax rvecs = {{__}} for single integrals, and rvecs = {{__},{__},..} for several integrals*)


Clear[diffop]

diffop[A_, beta_, sigma_, rvec_] /; Length@Dimensions@rvec > 1 := Module[{xs,dxs,strA,strAp,strbeta,strsigma,strrvec},

	{xs, dxs} = 
		simplexvars[A,sigma] // Map[asirlist] // StringDelete[#, ";"]&;
		
	{strA, strAp, strbeta, strsigma, strrvec} = 
		{A, A // Ap, beta, sigma, rvec} // Map[asirlist];
	
	StringJoin[
		"A = " <> strA <> "\n",
		"Ap = " <> strAp <> "\n",
		"Beta = " <> strbeta <> "\n",
		"Sigma = " <> strsigma <> "\n",
		"Rvec = " <> strrvec <> "\n",
		"NumDen = mt_gkz.rvec_red2(Rvec, A, Ap, Beta, Sigma);\n",
		"Num = map(mt_gkz.to_partial3, NumDen[0], " <> xs <> ");\n",
		"Num = map(dp_dtop, Num, " <> dxs <> ");\n",
		"Den = NumDen[1];\n",
		"NumDen = [];\n",
		"for (I=0; I<length(Num); I++) NumDen = append(NumDen, [Num[I]/Den[I]]);\n\n",
		"OUTPUT = NumDen;"
	]
]
