(* ::Package:: *)

(* ::Title:: *)
(*Assortment of useful functions*)
(*Authors: Seva and Vitaly *)


(* ::Section:: *)
(*Make files*)


(* ::Text:: *)
(*Make string*)


MkString[args__] := args // List // Flatten // Map[ToString] // StringJoin


(* ::Text:: *)
(*Make random string name*)


MkTemp[prefix_, suffix_] := Module[{i, fn, alphabet},
  alphabet = Characters["abcdefghijklmnopqrstuvwxyz0123456789"];
  While[True,
    i = RandomSample[alphabet, 8];
    fn = FileNameJoin[{MkString[prefix, ".", Environment["USER"], ".", $ProcessID, ".", i, suffix]}];
    If[Not[FileExistsQ[fn]], Return[fn]];
  ]
]


(* ::Text:: *)
(*Convert items into string and write into file object f*)


WrString[f_, items__] := {items} // Flatten // Map[BinaryWrite[f, # // ToString]&]


(* ::Text:: *)
(*Make file*)


MkFile[filename_, items__] := Module[{fd},
  (* The BinaryFormat is needed for the BinaryWrite in WrString. *)
  fd = OpenWrite[MkString[filename], BinaryFormat->True];
  If[fd === $Failed, Error["MkFile: failed to open ", filename, " for writing"]];
  WrString[fd, {items}];
  Close[fd];
]


(* ::Section:: *)
(*Misc*)


(* ::Text:: *)
(*Convert t123 --> t[1,2,3]*)


sym2ind = RightComposition[
    MkString,
    StringCases[#, Alternatives[LetterCharacter.., DigitCharacter..]]&,
    Map[ToExpression],
    Apply[Construct],
    (*Edge case of no arguments*)
    Replace[x_[] :> x],
    Identity
];


(* ::Text:: *)
(*Convert t[1,2,3] --> t123*)


ind2sym = MkString /* StringDelete[{"[", "]", ",", " "}] /* ToExpression;


(* ::Text:: *)
(*Timing*)


SetAttributes[TM, HoldFirst];
TM[ex_] := AbsoluteTiming[ex] // (Print[HoldForm[ex], ": ", #[[1]], " sec"]; #[[2]])&;
TM[str_String] := Function[ex, AbsoluteTiming[ex] // (Print[str, ": ", #[[1]], " sec"]; #[[2]])&, HoldFirst];


(* ::Text:: *)
(*Column with spacing*)


col[expr_] := Column[expr, Spacings -> 1.5] 


(* ::Text:: *)
(*Collect dx's*)


jsimp = Collect[#,j[__],Factor]&


(* ::Text:: *)
(*Multiply everything by xs*)


Multiply[xs__] := Times[xs, #]&;


(* ::Text:: *)
(*Isolate patterns*)


filter[pattern_] := Union @ Flatten @ Cases[#, pattern, -1]&;


(* ::Text:: *)
(*a+b+... to {a,b,...}*)


PlusToList[expr_Plus] := expr // Apply[List];
PlusToList[expr_] := expr // List;
PowerToList[Power[expr_, n_Integer]] := ConstantArray[expr^Sign[n], Abs[n]];
PowerToList[expr_] := expr // List;


(* ::Text:: *)
(*a*b*... to {a,b,...}*)


TimesToList[expr_Times] := expr // Apply[List] // Map[PowerToList] // Flatten;
TimesToList[expr_Power] := expr // PowerToList;
TimesToList[expr_] := expr // List;


(* ::Text:: *)
(*Denominators of a list*)


getDens[x_List] := x // RightComposition[
    Flatten,
    Together,
    Denominator,
    Factor,
    Map[TimesToList],
    Flatten,
    Union,
    DeleteCases[n_ /; NumericQ[n]],
    Expand,
    Map[RightComposition[
        PlusToList,
        # / (#[[1]] // Replace[(n_ | n_. _) /; NumericQ[n] :> n])&,
        Apply[Plus],
        Identity
    ]],
    Union,
    Identity
];
getDens[x_] := getDens[{x}];



(* ::Text:: *)
(*Nullspace*)


deleteZeroRows = Apply[List] /* DeleteCases[x_ /; SameQ[x["Density"], 0.]] /* SparseArray;
rowReduce = RowReduce /* SparseArray /* deleteZeroRows;
getNullSpace = NullSpace /* SparseArray;


(* ::Text:: *)
(*Independent rows*)


indrows[mat_List] := mat // RightComposition[
	Transpose,
	rowReduce,
	Transpose,
	Apply @ List,
	Position[x_ /; Length[x["NonzeroValues"]] === 1],
	Flatten,
	Rest
]


(* ::Text:: *)
(*Numerics*)


numrep[params_] := params // Rule[#, # // Length // RandomPrime[{10, 10^5}, #]^(-1) &]& // Thread // ReplaceAll


(* ::Text:: *)
(*Replace x \[RightArrow] a + b*t, where a and b are random primes*)


slice[xs_] := Module[{len},
	SeedRandom[1234];
	len = Length@xs;
	Rule[
		xs,
		Range[10^2] // Map[Prime] // RandomSample[#, 2*len]& // Partition[#, len]& // Inner[Times, # // Transpose, {1, t}]&
	] // Thread
]


(* ::Text:: *)
(*Replace denominators with den[i] using slicing*)


denrep[dens_, slice_] := Module[{reps},
	reps = Rule[
		dens /. slice // Expand, 
		den /@ Range @ Length @ dens
	] // Thread;
	Join[
		reps, 
		reps /. Rule[f_,den_] :> Rule[Expand[-f],-den]
	] // ReplaceAll
]


(* ::Text:: *)
(*Replace denominator in expr with denrep. *)
(*To do: Include Map[cleanden, expr, {2}]*)


cleanden[denrep_,expr_] := Module[{denreps},
	denreps = expr // RightComposition[
		TimesToList,
		Replace[#, 
			Power[x_,a_] /; a<0  :>  Power[x // Replace[denrep], a], 
		{1}]&,
		Apply[Times]
	]
]
